package pl.mpak.sky.gui.swing;

import java.awt.Toolkit;
import java.io.File;
import java.net.URL;
import java.util.HashMap;

import javax.swing.ImageIcon;

/**
 * Klasa adujca (o ile to konieczne) obrazki z zasobw programu
 * 
 * @author Andrzej Kaua
 *
 */
public final class ImageManager {
  
  private static HashMap<String, ImageIcon> imageList = new HashMap<String, ImageIcon>();
  
  /**
   * Zwraca obrazek z listy lub jeli jeszcze nie jest zaadowany aduje go
   * 
   * @param resName nazwa obrazka do pobrania
   * @return pobrany obrazek
   */
  public final static ImageIcon getImage(String resName) {
    return getImage(resName, Toolkit.getDefaultToolkit().getClass());
  }
  
  public final static ImageIcon getImage(String resName, Class<?> rootClass) {
    synchronized (imageList) {
      if (imageList.containsKey(resName)) {
        return imageList.get(resName);
      }
      else {
        try {
          URL url;
          File file = new File(resName);
          if (file.exists()) {
            url = file.toURI().toURL();
          }
          else {
            url = rootClass.getResource(resName);
          }
          ImageIcon ii = new ImageIcon(url);
          imageList.put(resName, ii);
          return ii;
        }
        catch (Throwable e) {
          imageList.put(resName, null);
        }
        return null;
      }
    }
  }
  
}
