package pl.mpak.sky.gui.swing.syntax;

import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.Segment;

import pl.mpak.sky.gui.swing.syntax.SyntaxDocument.LineInfo;
import pl.mpak.sky.gui.swing.syntax.SyntaxDocument.Token;

public class TokenCursor {

  private SyntaxDocument document;
  private int lineIndex;
  private int offset;
  private LineInfo lineInfo;
  private Token token;
  
  public TokenCursor(SyntaxDocument document) {
    this.document = document;
    startToken(0);
  }
  
  public Token startToken(int offset) {
    Segment line = new Segment();
    try {
      Element map = document.getDefaultRootElement();
      lineIndex = map.getElementIndex(offset);
      lineInfo = document.getLineInfoAtLine(lineIndex, line);
    } catch (BadLocationException e) {
      return null;
    }
    int lineOffset = document.getLineOffsetAt(offset);
    token = lineInfo.getTokenAtOffset(offset -lineOffset);
    if (token != null) {
      this.offset = lineOffset +token.offset;
    }
    return token;
  }
  
  public Token startToken(int offset, int styleId) {
    Token t = startToken(offset);
    while (t != null && t.styleId != styleId) {
      t = nextToken();
    }
    return t;
  }
  
  public String getString() {
    if (token != null) {
      try {
        return document.getText(offset, token.length);
      } catch (BadLocationException e) {
        return null;
      }
    }
    return null;
  }
  
  public Token getToken() {
    return token;
  }
  
  public Token nextToken() {
    if (token != null) {
      offset += token.length;
      if (token.next != null) {
        token = token.next;
      }
      else {
        Segment line = new Segment();
        lineIndex++;
        try {
          lineInfo = document.getLineInfoAtLine(lineIndex, line);
        } catch (BadLocationException e) {
          token = null;
          return null;
        }
        if (lineInfo != null && lineInfo.tokens.size() > 0) {
          token = lineInfo.tokens.get(0);
        }
        else {
          token = null;
        }
      }
    }
    return token;
  }
  
  public Token nextToken(int styleId) {
    Token t = nextToken();
    while (t != null && t.styleId != styleId) {
      t = nextToken();
    }
    return t;
  }

  public Token prevToken() {
    if (token != null) {
      if (token.prev != null) {
        token = token.prev;
        offset -= token.length;
      }
      else if (lineIndex > 0) {
        Segment line = new Segment();
        lineIndex--;
        try {
          lineInfo = document.getLineInfoAtLine(lineIndex, line);
        } catch (BadLocationException e) {
          token = null;
          return null;
        }
        if (lineInfo != null && lineInfo.tokens.size() > 0) {
          token = lineInfo.tokens.get(lineInfo.tokens.size() -1);
          offset -= token.length;
        }
        else {
          token = null;
        }
      }
      else {
        token = null;
      }
    }
    return token;
  }

  public Token prevToken(int styleId) {
    Token t = prevToken();
    while (t != null && t.styleId != styleId) {
      t = prevToken();
    }
    return t;
  }

}
